#!/usr/bin/perl
#  Assemble a source file in source (src) format with dap-16 to an
#  object file and list file
#  The source file can have only a single source file; if more than one source
#  file is present in the file, only the 1st one will be assembled.
#  The assembly is made as a 2-pass assembly
#
#  Use: dap-16.pl '<name>' (src extension is optional)
#
#  Assumed is that a file with name <name>.src exists in the current directory.
#  The .src file must be in ASCII code and with the record format accepted
#  by the assembler.

use strict;
# 
my $DEBUG=1;
#############################################
my $EMU='h316-370t'; # name of the emulator program (in working directory)
#############################################
# The current directory is the directory from which the Perl program is
# executed, together which the source files to be assembled.
# Also the result of the assembly: object and listing file, will copied
# to this directory.
my $CURDIR = `pwd`;
my $l = length($CURDIR);
$CURDIR = substr($CURDIR, 0, $l-1);
$CURDIR = $CURDIR . "\/";
#############################################
# The working directory is the directory in which the h316 simulation program lives,
# as well as its programs like assembler, loader, etc.
# Default is: the current directory
#my $WRKDIR = '/usr/local/src/simhv36-1/BIN/';
#my $WRKDIR = '/usr/local/src/simhv37-0t/BIN/';
my $WRKDIR = $CURDIR;
#############################################
print "Current directory: $CURDIR\n";
print "Working directory: $WRKDIR\n";
#
# $ARGV[0] contains the name of the source file to process.
my $ft = $ARGV[0];
# print "command $0 $ft\n";
my $fn = "";
my $ext = "";

if ($ft =~ /\./)
{
	($fn, $ext) = split(/\./,$ft);
}
else
{
	$fn = $ft;
	$ext = "src";
}
if( $fn eq "" || $ext ne "src")
{
	print "No filename, or wrong extension specified\n";
	exit;
}

#
my $src = $fn . "\." . "src";
my $obj = $fn . "\." . "obj";
my $lst = $fn . "\." . "lst";
print "The following files will be processed by a 2-pass assembly:\n";
print " $src is the source file to be assembled\n";
print " $obj will be the assembled object file\n";
print " $lst will contain the listing of the assembled source file\n\n";
#
# does $src exist?
if(-e $src)
{ }
else
{
	print "$src does not exist, so exit\n";
    exit;
}
if($CURDIR != $WRKDIR)
{
    # move the source file to the working directory
    my $copy = "cp " . "\'" . $src . "\' \'" . $WRKDIR . $src . "\'" . "\n";
    print $copy;
    `$copy`;
    # change to working direcotory
    chdir $WRKDIR || die;
}

# remove object and listing file to make, if they exist
if(-e $obj)
{
	unlink ($obj);
}
if(-e $lst)
{
	unlink ($lst);
}
#
# create the asm proc
my $proc = "proc";
open OUT, ">$proc";
print OUT "set console log=assembler.log\n";
print OUT "set cpu 16K\n";
print OUT "attach -b ptr dap16slst.ptp\n";    # ddp516 assembler
print OUT "boot ptr\n";
print OUT "do lptpatch\n";                    # patch the lpt driver of the assembler
print OUT "reset\n";
print OUT "attach -a ptr $src\n";
print OUT "attach -b ptp $obj\n";
print OUT "attach -a lpt $lst\n";
print OUT "\;a-reg with in=ptr binout=ptp list=lpt\n";
print OUT "de a 120402\n";
print OUT "ex a\n";
print OUT "de p 400\n";                       # start pass 1
print OUT "go\n";
print OUT "detach ptr\n";
print OUT "attach ptr -b $src\n";
print OUT "cont\n";                           # start pass 2
print OUT "quit\n";
close OUT;
my $ex = "./$EMU proc\n";
print $ex;
`$ex`;
#

if($CURDIR != $WRKDIR)
{
    # copy result to current directory
    # move the object file to the current directory
    my $copyobj = "cp " . "\'" . $obj . "\' \'" . $CURDIR . $obj . "\'" . "\n";
    print $copyobj;
    `$copyobj`;
    # move the listing file to the current directory
    my $copylst = "cp " . "\'" . $lst . "\' \'" . $CURDIR . $lst . "\'" . "\n";
    print $copylst;
    `$copylst`;
    # and remove the files from the working directory
    #unlink ($src);
    #unlink ($obj);
    #unlink ($lst);
    chdir $CURDIR || die;
}

print "Assembly of $src is ready\n";
open IN, $lst || die;
while (<IN>)
{
  if($_ =~ /ERRORS IN ABOVE ASSEMBLY/)
  {
    last;
  }
}

if($_ =~ /\*\*/)
{
  print "ERRORS in ASSEMBLY\n";
}
if($_ =~ /NO/)
{
  print "NO ERRORS in ASSEMBLY\n";
}
close IN || die;


